@@ -1,14 +1,10 @@
 # -*- coding: utf-8 -*-
-import requests
-
 import pydov.util.caching
-from pydov.util.hooks import (
-    SimpleStatusHook,
-    Hooks,
-)
+from pydov.util.hooks import Hooks, SimpleStatusHook
+from pydov.util.net import SessionFactory
 
 __author__ = """DOV-Vlaanderen"""
-__version__ = '2.0.0'
+__version__ = '2.1.0'
 
 cache = pydov.util.caching.GzipTextFileCache()
 
@@ -18,6 +14,4 @@ hooks = Hooks(
 
 # Package wide requests session object. This increases performance as using a
 # session object allows connection pooling and TCP connection reuse.
-request_timeout = 300
-session = requests.Session()
-session.headers.update({'user-agent': 'pydov/{}'.format(pydov.__version__)})
+session = SessionFactory.get_session()
