@@ -30,9 +30,13 @@ class AbstractCommonInterpretatie(AbstractDovType):
         WfsField(name='betrouwbaarheid_interpretatie',
                  source_field='Betrouwbaarheid', datatype='string'),
         WfsField(name='x', source_field='X_mL72', datatype='float'),
-        WfsField(name='y', source_field='Y_mL72', datatype='float')
+        WfsField(name='y', source_field='Y_mL72', datatype='float'),
+        WfsField(name='start_interpretatie_mtaw', source_field='Z_mTAW',
+                 datatype='float')
     ]
 
+    pkey_fieldname = 'Interpretatiefiche'
+
     def __init__(self, pkey):
         """Initialisation.
 
@@ -44,13 +48,13 @@ class AbstractCommonInterpretatie(AbstractDovType):
             `https://www.dov.vlaanderen.be/data/interpretatie/<id>`.
 
         """
-        super(AbstractCommonInterpretatie, self).__init__(
-            'interpretatie', pkey)
+        super().__init__('interpretatie', pkey)
 
     @classmethod
     def from_wfs_element(cls, feature, namespace):
         instance = cls(
-            feature.findtext('./{{{}}}Interpretatiefiche'.format(namespace)))
+            feature.findtext('./{{{}}}{}'.format(
+                namespace, cls.pkey_fieldname)))
 
         typeproef = cls._parse(
             func=feature.findtext,
@@ -105,9 +109,13 @@ class AbstractBoringInterpretatie(AbstractDovType):
         WfsField(name='betrouwbaarheid_interpretatie',
                  source_field='Betrouwbaarheid', datatype='string'),
         WfsField(name='x', source_field='X_mL72', datatype='float'),
-        WfsField(name='y', source_field='Y_mL72', datatype='float')
+        WfsField(name='y', source_field='Y_mL72', datatype='float'),
+        WfsField(name='start_interpretatie_mtaw', source_field='Z_mTAW',
+                 datatype='float')
     ]
 
+    pkey_fieldname = 'Interpretatiefiche'
+
     def __init__(self, pkey):
         """Initialisation.
 
@@ -119,13 +127,13 @@ class AbstractBoringInterpretatie(AbstractDovType):
             `https://www.dov.vlaanderen.be/data/interpretatie/<id>`.
 
         """
-        super(AbstractBoringInterpretatie, self).__init__(
-            'interpretatie', pkey)
+        super().__init__('interpretatie', pkey)
 
     @classmethod
     def from_wfs_element(cls, feature, namespace):
         instance = cls(
-            feature.findtext('./{{{}}}Interpretatiefiche'.format(namespace)))
+            feature.findtext('./{{{}}}{}'.format(
+                namespace, cls.pkey_fieldname)))
 
         for field in cls.get_fields(source=('wfs',)).values():
             instance.data[field['name']] = cls._parse(
