@@ -7,6 +7,8 @@ Filter Encoding 1.1 and GML 3.1.1.
 """
 import os
 from collections import OrderedDict
+from distutils.version import LooseVersion
+from io import BytesIO
 
 from owslib.etree import etree
 from owslib.fes import Or
@@ -24,6 +26,7 @@ class AbstractLocation(object):
     initialisation.
 
     """
+
     def get_element(self):
         """Return the GML representation of the location.
 
@@ -50,6 +53,7 @@ class AbstractLocationFilter(object):
     `set_geometry_column` method.
 
     """
+
     def set_geometry_column(self, geometry_column):
         """Set the name of the geometry column to query.
 
@@ -91,6 +95,7 @@ class AbstractBinarySpatialFilter(AbstractLocationFilter):
     Contains.
 
     """
+
     def __init__(self, type, location):
         """Initialise a Binary spatial filter.
 
@@ -131,6 +136,7 @@ class AbstractBinarySpatialFilter(AbstractLocationFilter):
 class Box(AbstractLocation):
     """Class representing a box location, also known as bounding box,
     envelope, extent."""
+
     def __init__(self, minx, miny, maxx, maxy, epsg=31370):
         """Initialise a Box.
 
@@ -191,6 +197,7 @@ class Box(AbstractLocation):
 
 class Point(AbstractLocation):
     """Class representing a point location."""
+
     def __init__(self, x, y, epsg=31370):
         """Initialise a Point.
 
@@ -229,6 +236,7 @@ class Point(AbstractLocation):
 class GmlObject(AbstractLocation):
     """Class representing a raw GML location, f.ex. gml:Surface or
     gml:MultiSurface."""
+
     def __init__(self, gml_element):
         """Initialise a GmlObject.
 
@@ -242,9 +250,9 @@ class GmlObject(AbstractLocation):
             string representation.
 
         """
-        if type(gml_element) is str:
+        if isinstance(gml_element, str):
             self.element = etree.fromstring(gml_element.encode('utf8'))
-        elif type(gml_element) is bytes:
+        elif isinstance(gml_element, bytes):
             self.element = etree.fromstring(gml_element)
         else:
             self.element = gml_element
@@ -259,6 +267,7 @@ class Equals(AbstractBinarySpatialFilter):
     A spatial Equals will return all points that are equal to another point.
 
     """
+
     def __init__(self, location):
         """Initialise an Equals filter.
 
@@ -281,6 +290,7 @@ class Disjoint(AbstractBinarySpatialFilter):
     A spatial Disjoint is the inverse of a spatial Intersects.
 
     """
+
     def __init__(self, location):
         """Initialise a Disjoint filter.
 
@@ -301,6 +311,7 @@ class Touches(AbstractBinarySpatialFilter):
     location: i.e. that are on the boundary but not inside.
 
     """
+
     def __init__(self, location):
         """Initialise a Touches filter.
 
@@ -321,6 +332,7 @@ class Within(AbstractBinarySpatialFilter):
     polygon or box location (i.e. are not on the boundary).
 
     """
+
     def __init__(self, location):
         """Initialise a Within filter.
 
@@ -343,6 +355,7 @@ class Intersects(AbstractBinarySpatialFilter):
     A spatial Intersects is the inverse of a spatial Disjoint.
 
     """
+
     def __init__(self, location):
         """Initialise an Intersects filter.
 
@@ -363,6 +376,7 @@ class WithinDistance(AbstractLocationFilter):
     distance of a certain location.
 
     """
+
     def __init__(self, location, distance, distance_unit='meter'):
         """Initialise a WithinDistance filter.
 
@@ -496,9 +510,9 @@ class GmlFilter(AbstractLocationFilter):
         """
         gml_tree = None
         try:
-            if type(self.gml) is str:
+            if isinstance(self.gml, str):
                 gml_tree = etree.fromstring(self.gml.encode('utf8'))
-            elif type(self.gml) is bytes:
+            elif isinstance(self.gml, bytes):
                 gml_tree = etree.fromstring(self.gml)
         except etree.ParseError as error:
             if os.path.isfile(self.gml):
@@ -564,3 +578,144 @@ class GmlFilter(AbstractLocationFilter):
 
     def toXML(self):
         return self.element.toXML()
+
+
+class GeometryFilter(GmlFilter):
+    """Class for construction a spatial filter expression from any Geometry
+    Fiona can convert to a GML 3.1.1 document.
+    """
+
+    def __init__(self, geometry, location_filter,
+                 location_filter_kwargs=None, combinator=Or):
+        """Initialise a spatial filter expression from a given Geometry.
+
+        Parameters
+        ----------
+        geometry : str
+            A path to a vector file on disk.
+        location_filter : class<AbstractLocationFilter>
+            Location filter to use for the geometries in the GML document.
+        location_filter_kwargs : dict, optional
+            Keyword-based arguments to pass to the `location_filter` on
+            initialisation (with the exception of the `location` parameter,
+            which is automatically parsed from the geomery). Can be skipped in
+            cases where the location_filter takes no extra arguments besides
+            location.
+        combinator : class<BinaryLogicOpType>, optional, defaults to Or
+            One of (Or, And) used to combine filters for different geometries
+            in the geometry document.
+
+        Raises
+        ------
+        ValueError
+            When no geometries could be parsed from the given geometry.
+        """
+        gml = self._parse_geometry(geometry)
+
+        super().__init__(gml, location_filter, location_filter_kwargs,
+                         combinator)
+
+    @staticmethod
+    def _parse_geometry(geometry):
+        """Checks the geometry and parse the file if possible.
+
+        Raises
+        ------
+        ValueError
+            When the file could not be parsed.
+        """
+        try:
+            import fiona
+
+            # Check if the GML driver and write mode are supported and check
+            # if version of Fiona supports GML 3.1.1 output (Fiona >= 1.8.18)
+            drivers = fiona.supported_drivers
+            if 'w' not in drivers.get('GML', '') or \
+                    LooseVersion(fiona.__version__) < LooseVersion('1.8.18'):
+                raise UserWarning('Please use module Fiona version 1.8.18 '
+                                  'or higher')
+        except ImportError:
+            raise ImportError('No module named fiona. GeometryFilter '
+                              'requires fiona to be installed.')
+        else:
+            # Open the geometry as a collection containing multiple records
+            with fiona.open(geometry, 'r') as c:
+                gml_blob = BytesIO()
+                # Assume the geometry is loaded from file
+                layer = os.path.basename(os.path.splitext(geometry)[0])
+                with fiona.open(gml_blob, 'w', layer=layer, driver='GML',
+                                schema=c.schema, crs=c.crs,
+                                FORMAT='GML3') as gml:
+                    for r in c:
+                        gml.write(r)
+                gml_blob.seek(0)
+                # Set the gml attribute used by the parent class
+                gml = gml_blob.read()
+                return gml
+
+
+class GeopandasFilter(GmlFilter):
+    """Class for construction a spatial filter expression from a
+    GeoPandas object.
+    """
+
+    def __init__(self, geodataframe, location_filter,
+                 location_filter_kwargs=None, combinator=Or):
+        """Initialise a spatial filter expression from a GeoPandas
+        GeoDataFrame.
+
+        Parameters
+        ----------
+        geodataframe : GeoPandas.GeoDataFrame
+            A GeoDataFrame with a valid crs.
+        location_filter : class<AbstractLocationFilter>
+            Location filter to use for the geometries in the GML document.
+        location_filter_kwargs : dict, optional
+            Keyword-based arguments to pass to the `location_filter` on
+            initialisation (with the exception of the `location` parameter,
+            which is automatically parsed from the geomery). Can be skipped in
+            cases where the location_filter takes no extra arguments besides
+            location.
+        combinator : class<BinaryLogicOpType>, optional, defaults to Or
+            One of (Or, And) used to combine filters for different geometries
+            in the geometry document.
+
+        Raises
+        ------
+        ValueError
+            When no geometries could be parsed from the given geometry.
+        """
+        gml = self._parse_geometry(geodataframe)
+
+        super().__init__(gml, location_filter, location_filter_kwargs,
+                         combinator)
+
+    @staticmethod
+    def _parse_geometry(gdf):
+        """Checks the geometry and parse the file if possible.
+
+        Raises
+        ------
+        ValueError
+            When the file could not be parsed.
+        """
+        try:
+            import geopandas
+        except ImportError:
+            raise ImportError('No module named GeoPandas. GeopandasFilter '
+                              'requires geopandas to be installed.')
+        else:
+            if not isinstance(gdf, geopandas.GeoDataFrame):
+                raise TypeError("pydov only supports geopandas.GeoDataFrame "
+                                "to define a spatial query.")
+
+            if not gdf.crs:
+                raise AttributeError("geopandas.GeoDataFrame is missing a "
+                                     "CRS definition")
+
+            with BytesIO() as gml_blob:
+                gdf.to_file(gml_blob, driver="GML", FORMAT="GML3",
+                            layer="geodataframe")
+                gml_blob.seek(0)
+                gml = gml_blob.read()
+            return gml
