@@ -19,36 +19,49 @@ from pydov.util.location import Box, Within
 from pydov.util.query import Join, PropertyInList
 
 
-def service_ok(timeout=5):
-    """Check whether DOV services are accessible.
+class ServiceCheck:
 
-    Used to skip online tests when the service is unavailable or unreachable.
+    is_service_ok = None
 
-    Parameters
-    ----------
-    timeout : int, optional
-        Timeout in seconds. Defaults to 5.
+    @staticmethod
+    def service_ok(timeout=5):
+        """Check whether DOV services are accessible.
 
-    Returns
-    -------
-    bool
-        True if the DOV services are reachable, False otherwise.
+        Used to skip online tests when the service is unavailable or
+        unreachable.
 
-    """
-    def check_url(url, timeout):
-        try:
-            ok = pydov.session.head(
-                url, allow_redirects=True, timeout=timeout).ok
-        except requests.exceptions.ReadTimeout:
-            ok = False
-        except requests.exceptions.ConnectTimeout:
-            ok = False
-        except Exception:
-            ok = False
-        return ok
+        Parameters
+        ----------
+        timeout : int, optional
+            Timeout in seconds. Defaults to 5.
 
-    return check_url(build_dov_url('geoserver'), timeout) and\
-        check_url(build_dov_url('geonetwork'), timeout)
+        Returns
+        -------
+        bool
+            True if the DOV services are reachable, False otherwise.
+
+        """
+        def check_url(url, timeout):
+            try:
+                ok = pydov.session.head(
+                    url, allow_redirects=True, timeout=timeout).ok
+            except requests.exceptions.ReadTimeout:
+                ok = False
+            except requests.exceptions.ConnectTimeout:
+                ok = False
+            except Exception:
+                ok = False
+            return ok
+
+        if ServiceCheck.is_service_ok is None:
+            ServiceCheck.is_service_ok = (
+                check_url(build_dov_url('geoserver'), timeout) and
+                check_url(build_dov_url('geonetwork'), timeout) and
+                check_url(
+                    build_dov_url('xdov/schema/latest/xsd/kern/dov.xsd'),
+                    timeout))
+
+        return ServiceCheck.is_service_ok
 
 
 def clean_xml(xml):
@@ -329,6 +342,9 @@ class AbstractTestSearch(object):
             Monkeypatch the call to get WFS features.
 
         """
+        if self.valid_returnfields_subtype is None:
+            return
+
         df = self.search_instance.search(
             query=self.valid_query_single,
             return_fields=self.valid_returnfields_subtype)
@@ -410,6 +426,9 @@ class AbstractTestSearch(object):
         Test whether an InvalidFieldError is raised.
 
         """
+        if self.xml_field is None:
+            return
+
         query = PropertyIsEqualTo(propertyname=self.xml_field,
                                   literal='Geotechnisch onderzoek')
 
@@ -489,6 +508,9 @@ class AbstractTestSearch(object):
             Monkeypatch the call to get the remote XML data.
 
         """
+        if self.xml_field is None:
+            return
+
         with pytest.raises(InvalidFieldError):
             self.search_instance.search(
                 query=self.valid_query_single,
@@ -734,6 +756,9 @@ class AbstractTestTypes(object):
         Test whether an InvalidFieldError is raised.
 
         """
+        if self.valid_returnfields_subtype is None:
+            return
+
         with pytest.raises(InvalidFieldError):
             self.datatype_class.get_field_names(
                 return_fields=self.valid_returnfields_subtype,
@@ -802,6 +827,9 @@ class AbstractTestTypes(object):
         Test whether fields provides by subtypes are not listed in the output.
 
         """
+        if self.field_names_subtypes is None:
+            return
+
         fields = self.datatype_class.get_fields(include_subtypes=False)
         for field in fields:
             assert field not in self.field_names_subtypes
@@ -823,10 +851,11 @@ class AbstractTestTypes(object):
 
         assert isinstance(feature, self.datatype_class)
 
-        assert feature.pkey.startswith(self.pkey_base)
+        if self.pkey_base is not None:
+            assert feature.pkey.startswith(self.pkey_base)
 
-        assert feature.pkey.startswith(
-            build_dov_url('data/{}/'.format(feature.typename)))
+            assert feature.pkey.startswith(
+                build_dov_url('data/{}/'.format(feature.typename)))
 
         assert isinstance(feature.data, dict)
         assert isinstance(feature.subdata, dict)
