@@ -1,16 +1,13 @@
 """Module grouping tests for the pydov.util.location.GmlFilter class."""
-from owslib.fes import (
-    And,
-    Or,
-)
-from pydov.util.location import (
-    GmlFilter,
-    WithinDistance,
-    Within,
-    Disjoint,
-    Box,
-)
+import pytest
+import platform
+import subprocess
 from owslib.etree import etree
+from owslib.fes import And, Or
+
+from pydov.util.location import (Box, Disjoint, GeometryFilter,
+                                 GeopandasFilter, GmlFilter, Within,
+                                 WithinDistance)
 from pydov.util.owsutil import set_geometry_column
 from tests.abstract import clean_xml
 
@@ -741,3 +738,153 @@ class TestCombination(object):
             '186910.000000</gml:lowerCorner><gml:upperCorner>112220.000000 '
             '202870.000000</gml:upperCorner></gml:Envelope></ogc'
             ':Within></ogc:Or>')
+
+
+class TestGeometryFilter(object):
+    """Class grouping tests for the GeometryFilter."""
+
+    def test_shapefile(self):
+        """Test the conversion of a shapefile to GML using fiona,
+        Within operator.
+
+        Test whether the generated XML is correct and stable.
+        """
+        shapefile = 'tests/data/util/location/polygon_multiple_31370.shp'
+
+        f = GeometryFilter(shapefile, Within)
+        f.set_geometry_column('geom')
+        xml = f.toXML()
+
+        assert clean_xml(etree.tostring(xml).decode('utf8')) == clean_xml(
+            '<ogc:Or xmlns:gml311="http://www.opengis.net/gml" '
+            'xmlns:ogc="http://www.opengis.net/ogc"><ogc:Within><ogc:'
+            'PropertyName>geom</ogc:PropertyName><gml311:Polygon srsName='
+            '"urn:ogc:def:crs:EPSG::31370"><gml311:exterior>'
+            '<gml311:LinearRing><gml311:posList>108636.150020818 '
+            '194960.844295764 109195.573506438 195118.42837622 '
+            '108911.922161617 194291.111953824 108636.150020818 '
+            '194960.844295764</gml311:posList></gml311:LinearRing>'
+            '</gml311:exterior></gml311:Polygon></ogc:Within><ogc:Within>'
+            '<ogc:PropertyName>geom</ogc:PropertyName><gml311:Polygon '
+            'srsName="urn:ogc:def:crs:EPSG::31370"><gml311:exterior>'
+            '<gml311:LinearRing><gml311:posList>107485.786233486 '
+            '196741.544404921 108297.344247837 196843.974057217 '
+            '107840.350414513 196339.704999757 107485.786233486 '
+            '196741.544404921</gml311:posList></gml311:LinearRing>'
+            '</gml311:exterior></gml311:Polygon></ogc:Within></ogc:Or>')
+
+    def test_geopackage(self):
+        """Test the conversion of a geopackage to GML using fiona,
+        Within operator.
+
+        Test whether the generated XML is correct and stable.
+        """
+        gpkg = 'tests/data/util/location/polygon_multiple_31370.gpkg'
+
+        f = GeometryFilter(gpkg, Within)
+        f.set_geometry_column('geom')
+        xml = f.toXML()
+
+        assert clean_xml(etree.tostring(xml).decode('utf8')) == clean_xml(
+            '<ogc:Or xmlns:ogc="http://www.opengis.net/ogc"><ogc:Within>'
+            '<ogc:PropertyName>geom</ogc:PropertyName><gml:Polygon '
+            'xmlns:gml="http://www.opengis.net/gml" '
+            'srsName="urn:ogc:def:crs:EPSG::31370"><gml:exterior>'
+            '<gml:LinearRing><gml:posList>108636.150020818 194960.844295764 '
+            '108911.922161617 194291.111953824 109195.573506438 '
+            '195118.42837622 108636.150020818 194960.844295764</gml:posList>'
+            '</gml:LinearRing></gml:exterior></gml:Polygon></ogc:Within>'
+            '<ogc:Within><ogc:PropertyName>geom</ogc:PropertyName>'
+            '<gml:Polygon xmlns:gml="http://www.opengis.net/gml" '
+            'srsName="urn:ogc:def:crs:EPSG::31370"><gml:exterior>'
+            '<gml:LinearRing><gml:posList>107485.786233486 196741.544404921 '
+            '107840.350414513 196339.704999757 108297.344247837 '
+            '196843.974057217 107485.786233486 196741.544404921</gml:posList>'
+            '</gml:LinearRing></gml:exterior></gml:Polygon></ogc:Within>'
+            '</ogc:Or>')
+
+
+class TestGeopandasFilter:
+
+    def test_geopandas_dataframe(self):
+        """Test the conversion of a GeoPandas GeoDataFrame
+
+        Test whether the generated XML is correct and stable.
+        """
+        shapefile = 'tests/data/util/location/polygon_multiple_31370.shp'
+        import geopandas as gpd
+        gdf = gpd.read_file(shapefile)
+
+        f = GeopandasFilter(gdf, Within)
+        f.set_geometry_column('geom')
+        xml = f.toXML()
+
+        assert clean_xml(etree.tostring(xml).decode('utf8')) == clean_xml(
+            '<ogc:Or xmlns:gml311="http://www.opengis.net/gml" '
+            'xmlns:ogc="http://www.opengis.net/ogc"><ogc:Within><ogc:'
+            'PropertyName>geom</ogc:PropertyName><gml311:Polygon srsName='
+            '"urn:ogc:def:crs:EPSG::31370"><gml311:exterior>'
+            '<gml311:LinearRing><gml311:posList>108636.150020818 '
+            '194960.844295764 109195.573506438 195118.42837622 '
+            '108911.922161617 194291.111953824 108636.150020818 '
+            '194960.844295764</gml311:posList></gml311:LinearRing>'
+            '</gml311:exterior></gml311:Polygon></ogc:Within><ogc:Within>'
+            '<ogc:PropertyName>geom</ogc:PropertyName><gml311:Polygon '
+            'srsName="urn:ogc:def:crs:EPSG::31370"><gml311:exterior>'
+            '<gml311:LinearRing><gml311:posList>107485.786233486 '
+            '196741.544404921 108297.344247837 196843.974057217 '
+            '107840.350414513 196339.704999757 107485.786233486 '
+            '196741.544404921</gml311:posList></gml311:LinearRing>'
+            '</gml311:exterior></gml311:Polygon></ogc:Within></ogc:Or>')
+
+    def test_geopandas_dataframe_subset(self):
+        """Test the conversion of a GeoPandas GeoDataFrame subset.
+
+        Test whether the generated XML is correct and stable.
+        """
+        shapefile = 'tests/data/util/location/polygon_multiple_31370.shp'
+        import geopandas as gpd
+        gdf = gpd.read_file(shapefile)
+
+        # geopandas subselection to single feature line geodataframe
+        f = GeopandasFilter(gdf.iloc[[0]], Within)
+        f.set_geometry_column('geom')
+        xml = f.toXML()
+
+        assert clean_xml(etree.tostring(xml).decode('utf8')) == clean_xml(
+            '<ogc:Within xmlns:gml311="http://www.opengis.net/gml" '
+            'xmlns:ogc="http://www.opengis.net/ogc"><ogc:PropertyName>'
+            'geom</ogc:PropertyName><gml311:Polygon srsName="urn:ogc:def:crs:'
+            'EPSG::31370"><gml311:exterior><gml311:LinearRing><gml311:posList>'
+            '108636.150020818 194960.844295764 109195.573506438 '
+            '195118.42837622 108911.922161617 194291.111953824 '
+            '108636.150020818 194960.844295764</gml311:posList>'
+            '</gml311:LinearRing></gml311:exterior></gml311:Polygon>'
+            '</ogc:Within>')
+
+    def test_geopandas_series(self):
+        """Test GeoPandas GeoSeries not supported by pydov spatial filter
+        """
+        shapefile = 'tests/data/util/location/polygon_multiple_31370.shp'
+        import geopandas as gpd
+        gdf = gpd.read_file(shapefile)
+
+        # geopandas geometry not supported Type
+        with pytest.raises(TypeError):
+            GeopandasFilter(gdf.geometry, Within)
+
+        # geopandas series not supported Type
+        with pytest.raises(TypeError):
+            GeopandasFilter(gdf.iloc[0, :], Within)
+
+    def test_geopandas_nocrs(self):
+        """Test GeoPandas GeoSeries not supported by pydov spatial filter
+        """
+        shapefile = 'tests/data/util/location/polygon_multiple_31370.shp'
+        import geopandas as gpd
+        gdf = gpd.read_file(shapefile)
+        gdf.crs = None
+
+        # geopandas geometry not supported Type
+        with pytest.raises(AttributeError):
+            GeopandasFilter(gdf, Within)
